/*
 * file VersionHistory.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException.ReasonCode;


/**
 * A proxy for a version history resource.
 * 
 * A version history resource contains all versions of a version-controlled resource
 * as the bound members of the version history.
 * The binding name of a version in a version history is the version name of that version.
 * When a version is removed as a bound member from its version history,
 * it is also removed from any other folder that contains it as a bound member.
 * 
 * @since 1.0
 */
public interface VersionHistory extends Folder {

    /**
     * Get the workspace provider of this resource.
     * 
     * @return the {@link WorkspaceProvider} for this Resource.
     */
    public WorkspaceProvider workspaceProvider();

    /**
     * The root version of this VersionHistory.
     * The root version has no predecessors,
     * and every other version in this version history
     * is a descendant of the root version.
     */
    public static final PropertyName<Version> ROOT_VERSION =
        new PropertyName<Version>("root-version"); //$NON-NLS-1$

    /**
     * Get the {@link #ROOT_VERSION} property.
     * 
     * @return the {@link #ROOT_VERSION} property.
     * @throws WvcmException if this VersionHistory was not created with
     * {@link #ROOT_VERSION} as a wanted property.
     */
    public Version getRootVersion() throws WvcmException;

    /**
     * Get the latest Version for a given activity.
     * 
     * @param activity the activity whose latest product is to be returned.
     * @param feedback the properties available in the returned proxies.
     * @return the latest Version in this VersionHistory
     * that is a product of the specified activity. May be <code>null</code>.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This resource does not support this report.
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     *  The activity argument MUST identify an activity.  
     */
    public Version doLatestActivityVersionReport(
            Activity activity,
            Feedback feedback)
    throws WvcmException;

}
